#! /usr/bin/env python

import traceback
import logging

# process a node and assemble the info to build a scene, 
# currently only processes scenes with a flag of 132

logger = logging.getLogger(__name__)

class Scene_Info(object):

    def __init__(self,group):

        self.valid = False

        try:

            self.flag = group.attrib ['flag']
            self.id = group.attrib['nodeDefId']

            address = group.find('address')
            self.address = address.text

            name = group.find('name')
            self.name = name.text

            self.family = group.find('family').text

            if self.flag == '12': # ignore root scene
                logger.info('ignoring root scene: {}'.format(self.name))
                return 
           
            self.primary_node = group.find('pnode').text 
            
            self.device_group = group.find('deviceGroup').text

            self.controllers = [] # list of controller addresses
            self.responders = [] # list of responder addresses
            
            members = group.find('members')
            for link in members.iterfind('link'):
                link_type = link.attrib['type']
                if link_type == '0': # responder
                    self.responders.append(link.text)
                elif link_type == '16': # controller
                    self.controllers.append(link.text)

            self.valid = True

        except Exception as ex:
                #print ('scene info error',ex)
                traceback.print_exc()

    def __repr__(self):
        return 'Scene: Name {} Address {}, Family {}, Flag {}, Controllers {}, Responders {}'.format(self.name,self.address,self.family,self.flag,self.controllers,self.responders)



#! /usr/bin/env python


from ..common.device_switch import Device_Switch
from .device_zwave_base import Device_ZWave_Base

paddle_events = {'DON','DOF','DIM','BRT','DFON','DFOF'}

class Device_ZWave_GenericRspCtl(Device_Switch,Device_ZWave_Base):

    def __init__(self, container, device_info):
        Device_Switch.__init__(self,container,device_info.name,device_info.address)
        Device_ZWave_Base.__init__(self,device_info)

        self.add_property('paddle_action')

        value = device_info.get_property('ST','value')
        if value:
            try:
                if int(value) > 0:
                    self.set_property('onoff','on')
                else:
                    self.set_property('onoff','off')
            except:
                pass

    def process_websocket_event(self,event):
            if event.control == 'ST':
                if int(event.action) > 0:
                    self.set_property('onoff','on')
                else:
                    self.set_property('onoff','off')

            elif event.control in paddle_events: #need to add other events
                self.set_property('paddle_action',event.control,True)

    def turn_on(self):
        path = ('nodes/' + self.address + '/cmd/DON')
        return self.send_request(path)

    def turn_off(self):
        path = ('nodes/' + self.address + '/cmd/DOF')
        return self.send_request(path)



#! /usr/bin/env python



from ..common.device_base import Device_Base
from .device_zwave_base import Device_ZWave_Base

paddle_events = {'DON','DOF','DIM','BRT','DFON','DFOF'}

class Device_ZWave_Controller(Device_Base,Device_ZWave_Base):

    def __init__(self, container, device_info):
        Device_Base.__init__(self,container,'controller',device_info.name,device_info.address)
        Device_ZWave_Base.__init__(self, device_info)

        self.add_property('paddle_action')

    def process_websocket_event(self,event):
        if event.control in paddle_events: 
            self.set_property('paddle_action',event.control,True) 


#!/usr/local/bin/python

import asyncio
import aiohttp
from websocket_event import Websocket_Event
import xml.etree.ElementTree as ET
import logging
logger = logging.getLogger(__name__)

headers = {
    "Sec-WebSocket-Protocol": "ISYSUB",
    "Origin" : "com.universal-devices.websockets.isy",
}


URL = "ws://"+'192.168.1.51'+"/rest/subscribe"


class Websocket_Client(object):

    def __init__(self,controller,address,port,username,password,https):
        self._address = address
        self._port = port
        self.controller = controller
        self._https = https
        self.auth = aiohttp.BasicAuth('admin','admin')
        self._connected = False
    
    @property
    def connected(self):
        return self._connected

    @connected.setter
    def connected(self,connected):
        self._connected = connected
        if self.controller:
            self.controller.websocket_connected(connected)

    async def connect(self):
        URL = "ws://"+self._address+"/rest/subscribe"
        session = aiohttp.ClientSession()
        async with session.ws_connect(URL,headers=headers,auth=self.auth) as ws:

            async for msg in ws:
                #print('Message received from server:', msg)
                #logger.debug('Websocket Message: {}'.format(message))
                if msg.type == aiohttp.WSMsgType.TEXT:
                    try:
                        event_node = ET.fromstring (msg.data)        
                        
                        if event_node.tag == 'Event':
                            event = Websocket_Event(event_node)

                            if event.valid:
                                print (event)
                                if self.controller:
                                    self.controller.websocket_event(event)
                    
                    except Exception as ex:
                        logger.error('Websocket On Message Error {}'.format(ex))
'''
                if msg.type == aiohttp.WSMsgType.TEXT:
                    if msg.data == 'close cmd':
                        await ws.close()
                        break
                    else:
                        await ws.send_str(msg.data + '/answer')
                elif msg.type == aiohttp.WSMsgType.ERROR:
                    break
'''


if __name__ == '__main__':
    try:
        wsc = Websocket_Client(False,"192.168.1.51",80,'admin','admin',False)

        loop = asyncio.get_event_loop()
        loop.run_until_complete(wsc.connect())

    except KeyboardInterrupt:
        print("KeyboardInterrupt has been caught.")



#! /usr/bin/env python


from .device_base import Device_Base



class Device_Thermostat(Device_Base):

    def __init__(self, container, name, address):
        Device_Base.__init__(self,container,'thermostat', name, address)

        self.add_property('temperature',0) 
        self.add_property('humidity',0) 
        self.add_property('coolsetpoint',0) 
        self.add_property('heatsetpoint',0) 
        self.add_property('systemmode',0) 
        self.add_property('fanmode',0) 

    def set_mode (self,mode):
        pass
 
    def set_heatsetpoint (self,setpoint):
        pass
 
    def set_coolsetpoint (self,setpoint):
        pass

    def set_fanmode (self,mode):
        pass


#! /usr/bin/env python


from ..common.device_power import Device_Power
from .device_zwave_base import Device_ZWave_Base

class Device_ZWave_Power(Device_Power,Device_ZWave_Base):

    def __init__(self, container, device_info):
        Device_Power.__init__(self,container,device_info.name,device_info.address)
        Device_ZWave_Base.__init__(self,device_info)

        value = device_info.get_property('ST','value')
        if value:
            try:
                self.set_property('power',value)
            except:
                pass

        # could capture other properties -> uofm, prec, format

    def process_websocket_event(self,event):
            if event.control == 'ST':
                self.set_property('power',event.action)


#! /usr/bin/env python


from ..common.device_lock import Device_Lock
from .device_zwave_base import Device_ZWave_Base

class Device_ZWave_Lock(Device_Lock,Device_ZWave_Base):

    def __init__(self, container, device_info):
        Device_Lock.__init__(self,container,device_info.name,device_info.address)
        Device_ZWave_Base.__init__(self,device_info)

        value = device_info.get_property('ST','value')
        if value:
            try:
                if int(value) > 0:
                    self.set_property('state','Locked')
                else:
                    self.set_property('state','Unlocked')
            except:
                pass 

    def process_websocket_event(self,event):

        if event.control == 'ST':
            state_map = {0:'Unlocked', 100:'Locked'}
            self.set_property('state',state_map[int(event.action)])
        elif event.control == 'BATLVL':
            self.set_property('batterylevel',int(event.action))
        elif event.control == 'ERR':
            self.set_property('error',int(event.action))
        elif event.control == 'USRNUM':
            self.set_property('usernumber',int(event.action))


    def lock (self):
        path = ('nodes/' + self.address + '/cmd/SECMD/1')
        return self.send_request(path)

    def unlock (self):
        path = ('nodes/' + self.address + '/cmd/SECMD/0')
        return self.send_request(path)



#! /usr/bin/env python


from .device_base import Device_Base



class Device_Lock(Device_Base):

    def __init__(self, container, name, address):
        Device_Base.__init__(self,container,'lock', name, address)

        self.add_property('batterylevel',0) 
        self.add_property('state',0)
        self.add_property('error',0)
        self.add_property('usernumber',0) 

    def lock (self):
        pass

    def unlock (self):
        pass



#! /usr/bin/env python

''' 

Base Device

Common to all elements returned from rest/nodes/devices

device_type = switch,dimmer,contact,...

'''


from ... item_base import Item_Base


class Device_Base(Item_Base):

    def __init__(self, container, device_type, name, address):
        Item_Base.__init__(self,container, name)

        self.device_type = device_type
        self.address = address

    def __str__(self):
        return ('Device {}, type {}, ID {}'.format(self.name,self.device_type,self.address))

    def get_identifier(self):
        return self.address   
        


import asyncio

import aiohttp
#work in progress


class HTTP_Client(object):

    def __init__(self,address,port=None,username='',password='',use_https=False):
        self._address = address
        self._port = port
        self._username = username
        self._password = password

        url = (use_https and 'https://' or 'http://') + self._address + '/rest/'  
        # add port...
        self._url = url
        self.session = None

    async def get_session(self):
        async with aiohttp.ClientSession(
                auth=aiohttp.BasicAuth('admin', 'admin')) as self.session:
            await self.fetch()

    async def request(self,path):
        if not self.session is None:
            self.get_session()

        async with self.session.get(self._url + path) as resp:
            print(resp.status)
            body = await resp.text()
            print(body)


http = HTTP_Client('192.168.1.51')


loop = asyncio.get_event_loop()
loop.run_until_complete(http.request('nodes'))


#! /usr/bin/env python
import xml.etree.ElementTree as ET

from .. item_container import Item_Container
from .variable_info import Variable_Info
from .variable_name import Variable_Name
from .variable_integer import Variable_Integer
from .variable_state import Variable_State

import logging
logger = logging.getLogger(__name__)

variable_classes = {
    '1' : Variable_Integer,
    '2' : Variable_State,
} 

class Variable_Container (Item_Container):

    def __init__(self, controller):
        Item_Container.__init__(self,controller,'Variable')

    def get_list(self, request_string):
        success, response = self.controller.send_request(request_string)
        if not success or response.status_code != 200:
            raise Exception('no response for request: ', request_string)
        return ET.fromstring(response.content)

    def get_and_process(self, var_type):
        try:
            vars_list = self.get_list('/vars/get/' + var_type)
            defs_list = self.get_list('/vars/definitions/' + var_type)
            self.process_variable_nodes(vars_list, defs_list)
            
        except Exception as error:
            logger.warning('no variables found of type {}: {}'.format(var_type, error))
        
    def start(self):
        #success = True
        self.get_and_process('1')  # integer variables
        self.get_and_process('2')  # state variables
        self.items_retrieved = True
        return True
    
    def process_variable_nodes(self,list_root,name_root):
        for node in list_root.iter('var'):
            variable_info = Variable_Info(node)

            if variable_info.valid: # make sure we have the info we need
                variable_class = variable_classes [variable_info.type]
                variable = variable_class(self,variable_info)

                #find name
                for node in name_root.iter('e'):
                    variable_name = Variable_Name (node)

                    if variable_name.valid and variable_name.id == variable.id:
                        variable.name = variable_name.name

                self.add(variable,variable.get_index())
         
    def websocket_event(self,event):
        var = event.event_info_node.find('var')
        variable_id = var.attrib ['id']
        variable_type = var.attrib ['type']

        index = variable_type+':'+variable_id

        variable = self.get(index)
        if variable is not None:
            variable.process_websocket_event(event)
        else:
            logger.warning('Unable able to find variable type {} id {}'.format(variable_id,variable_type))




#! /usr/bin/env python


from .device_base import Device_Base


class Device_Power(Device_Base):

    def __init__(self, container, name , address):
        Device_Base.__init__(self,container,'power',name,address)

        self.add_property('power') 




#! /usr/bin/env python


from ..common.device_thermostat import Device_Thermostat
from .device_insteon_base import Device_Insteon_Base

MODES = ['Off','Heat','Cool','Auto','Program Auto']

FAN_MODES = {
    0 : 'Off',
    7 : 'On',
    8 : 'Auto',
}

class Device_Insteon_TempLinc(Device_Thermostat,Device_Insteon_Base):

    def __init__(self, container, device_info):
        Device_Thermostat.__init__(self,container,device_info.name,device_info.address)
        Device_Insteon_Base.__init__(self, device_info)

        coolsetpoint_address_parts = device_info.address_parts.copy()
        coolsetpoint_address_parts [3] = '2'
        self.coolsetpoint_address = ' '.join(coolsetpoint_address_parts)

        heatsetpoint_address_parts = device_info.address_parts.copy()
        heatsetpoint_address_parts [3] = '2'
        self.heatsetpoint_address = ' '.join(heatsetpoint_address_parts)

        #print ('device properties {}'.format(device_info.properties))
        value = device_info.get_property('ST','value')
        if value:
            try:       
                self.set_property('temperature',round(float(value)/2,0))
            except:
                self.set_property('temperature',0)
        
        value = device_info.get_property('CLIMD','value')
        if value:
            try:
                self.set_property('systemmode',MODES [int(value)])
            except:
                self.set_property('systemmode',0)

        value = device_info.get_property('CLISPH','value')
        if value:
            try:
                self.set_property('heatsetpoint',round(float(value)/2,0))
            except:
                self.set_property('heatsetpoint',0)

        value = device_info.get_property('CLISPC','value')
        if value: 
            try:
                self.set_property('coolsetpoint',round(float(value)/2,0))
            except:
                self.set_property('coolsetpoint',0)

        value = device_info.get_property('CLIHUM','value')
        if value:
            try:
                self.set_property('humidity',round(float(value),0))
            except:
                self.set_property('humidity',0)

        value = device_info.get_property('CLIFS','value')
        if value:
            try:
                self.set_property('fanmode',FAN_MODES [int(value)])
            except:
                self.set_property('humidity',0)

    def process_websocket_event(self,event):
        if event.control == 'ST':
            self.set_property('temperature',round(float(event.action)/2,0))
        elif event.control == 'CLISPH':
            self.set_property('heatsetpoint',round(float(event.action)/2,0))
        elif event.control == 'CLISPC':
            self.set_property('coolsetpoint',round(float(event.action)/2,0))
        elif event.control == 'CLIHUM':
            self.set_property('humidity',round(float(event.action),0))
        elif event.control == 'CLIMD':
            self.set_property('systemmode',MODES[int(event.action)])
        elif event.control == 'CLIFS':
            self.set_property('fanmode',FAN_MODES[int(event.action)])

    def set_mode (self,mode):
        path = ('nodes/' + self.address + '/cmd/DON/' + str(mode))
        return self.send_request(path)
 
    def set_heatsetpoint (self,setpoint):
        path = ('nodes/' + self.heatsetpoint_address + '/cmd/DON/' + str(setpoint*2))
        return self.send_request(path)
 
    def set_coolsetpoint (self,setpoint):
        path = ('nodes/' + self.coolsetpoint_address + '/cmd/DON/' + str(setpoint*2))
        return self.send_request(path)
 



#! /usr/bin/env python


from .. common.device_contact import Device_Contact
from .device_insteon_base import Device_Insteon_Base
 

class Device_Insteon_Contact(Device_Contact,Device_Insteon_Base):

    def __init__(self, container, device_info):
        Device_Contact.__init__(self,container,device_info.name,device_info.address)
        Device_Insteon_Base.__init__(self, device_info)

        value = device_info.get_property('ST','value')
        if value:
            try:
                if int(value) > 0:
                    self.set_property('contact','open')
                else:
                    self.set_property('contact','closed')
            except:
                pass


    def process_websocket_event(self,event):
            if event.control == 'ST':
                if int(event.action) > 0:
                    self.set_property('contact','open')
                else:
                    self.set_property('contact','closed')



#! /usr/bin/env python

''' 

returns a device instance using node data from an insteon device, None if unable to create device

'''

from .device_insteon_contact import Device_Insteon_Contact
from .device_insteon_dimmer import Device_Insteon_Dimmer
from .device_insteon_switch import Device_Insteon_Switch
from .device_insteon_fan import Device_Insteon_Fan
from .device_insteon_controller import Device_Insteon_Controller
from .device_insteon_templinc import Device_Insteon_TempLinc

insteon_device_classes = {
    '0'  : Device_Insteon_Controller,
    '1'  : Device_Insteon_Dimmer,
    '2'  : Device_Insteon_Switch,
    '14' : Device_Insteon_Switch,
    '16' : Device_Insteon_Contact,
}

'''
dev_cat_sub_cat = {
    '5' : {
        '10' : Device_Insteon_TempLinc,
    },
}
'''

def get_insteon_device_class (device_info):

    #print ('Insteon Device Class')
    #look for specific device dev/sub cat that need special handling
    #print(device_info)
    if device_info.category == '1' and device_info.sub_category == '46' and device_info.address_parts [3] == '2': # fanlinc motor
        return Device_Insteon_Fan

    if device_info.category == '5' and device_info.sub_category == '10' and device_info.address_parts [3] == '1': # temp linc
        return Device_Insteon_TempLinc

    if device_info.category == '7' and device_info.sub_category == '0' and device_info.address_parts [3] == '1': # IOLinc sensor
        return Device_Insteon_Contact

    if device_info.category == '7' and device_info.sub_category == '0' and device_info.address_parts [3] == '2': # IOLinc relay
        return Device_Insteon_Switch

    #print (device_info.node_def_id.find('KeypadButton') , device_info.address_parts [3])
    #override device cat for keypadlinc dimmer buttons and change to switch type devices
    if device_info.node_def_id.find('KeypadButton') == 0 and device_info.address_parts [3] != '1': # maybe use node flag
        device_info.category = '2'
      
    #find device class
    #check for specific devcat/subcat

    #print (device_info,device_info.category)

    if device_info.category in insteon_device_classes:
        device_class = insteon_device_classes [device_info.category]
        return device_class

'''        
    if device_info.category in dev_cat_sub_cat and device_info.sub_category in dev_cat_sub_cat [device_info.category]:
        device_class = dev_cat_sub_cat [device_info.category] [device_info.sub_category]
        return device_class
'''
